@@ -30,8 +30,8 @@ import siputils
 
 
 # Initialise the globals.
-sip_version = 0x041009
-sip_version_str = "4.16.9"
+sip_version = 0x041201
+sip_version_str = "4.18.1"
 py_version = sys.hexversion >> 8
 py_platform = sys.platform
 plat_py_site_dir = None
@@ -46,6 +46,7 @@ sip_bin_dir = ''
 sip_inc_dir = ''
 sip_module_dir = ''
 sip_sip_dir = ''
+pyi_dir = ''
 sysroot = ''
 src_dir = os.path.dirname(os.path.abspath(__file__))
 sip_module_base = None
@@ -158,7 +159,9 @@ def set_build_platform():
     build_platform = "none"
 
     if py_platform == "win32":
-        if py_version >= 0x030300:
+        if py_version >= 0x030500:
+            build_platform = "win32-msvc2015"
+        elif py_version >= 0x030300:
             build_platform = "win32-msvc2010"
         elif py_version >= 0x020600:
             build_platform = "win32-msvc2008"
@@ -181,6 +184,9 @@ def inform_user():
 
     siputils.inform("The %s module will be installed in %s." % (sip_module_base, sip_module_dir))
 
+    if opts.pyi:
+        siputils.inform("The sip.pyi stub file will be installed in %s." % pyi_dir)
+
     if opts.static:
         siputils.inform("The %s module will be built as a static library." % sip_module_base)
 
@@ -349,7 +355,7 @@ def create_makefiles(macros):
         pro.write("TEMPLATE = app\n")
         pro.write("TARGET = sip\n")
         pro.write("CONFIG -= qt app_bundle\n")
-        pro.write("CONFIG += warn_off exceptions_off console %s\n" % (
+        pro.write("CONFIG += warn_on exceptions_off console %s\n" % (
                 ("debug" if opts.debug else "release")))
 
         pro.write("\n")
@@ -378,7 +384,7 @@ def create_makefiles(macros):
             dir="sipgen",
             install_dir=os.path.dirname(cfg.sip_bin),
             console=1,
-            warnings=0,
+            warnings=1,
             universal=opts.universal,
             arch=opts.arch,
             deployment_target=opts.deployment_target
@@ -437,6 +443,12 @@ macx {
         pro.write("target.path = %s\n" % cfg.sip_mod_dir)
         pro.write("INSTALLS += target\n")
 
+        if opts.pyi:
+            pro.write("\n")
+            pro.write("sip_pyi.files = sip.pyi\n")
+            pro.write("sip_pyi.path = %s\n" % pyi_dir)
+            pro.write("INSTALLS += sip_pyi\n")
+
         pro.write("\n")
         pro.write("sip_h.files = sip.h\n")
         pro.write("sip_h.path = %s\n" % cfg.sip_inc_dir)
@@ -457,15 +469,21 @@ macx {
 
         build_dir = os.getcwd()
 
+        installs=[
+                ([os.path.join(build_dir, "siplib", "sip.h")],
+                        cfg.sip_inc_dir)]
+
+        if opts.pyi:
+            installs.append(([os.path.join(src_dir, 'sip.pyi')], pyi_dir))
+
         makefile = sipconfig.ModuleMakefile(
             configuration=cfg,
             build_file=os.path.join(build_dir, "siplib", "siplib.sbf"),
             dir="siplib",
             install_dir=cfg.sip_mod_dir,
-            installs=([os.path.join(build_dir, "siplib", "sip.h")],
-                    cfg.sip_inc_dir),
+            installs=installs,
             console=1,
-            warnings=0,
+            warnings=1,
             static=opts.static,
             debug=opts.debug,
             universal=opts.universal,
@@ -574,7 +592,7 @@ def update_from_configuration_file(config_file):
     config['py_minor'] = version[1]
     config['sysroot'] = sysroot
 
-    # Override the relevent values.
+    # Override the relevant values.
     global py_platform, plat_py_conf_inc_dir, plat_py_inc_dir, plat_py_lib_dir
     global sip_bin_dir, sip_inc_dir, sip_module_dir, sip_sip_dir
 
@@ -711,6 +729,12 @@ def create_optparser(sdk_dir):
             metavar="DIR", dest="sipsipdir", callback=store_abspath,
             help="where .sip files are normally installed [default: "
                     "%s]" % plat_sip_dir)
+    g.add_option("--no-pyi", action="store_false", default=True, dest="pyi",
+            help="do not install the sip.pyi stub file")
+    g.add_option("--pyidir", action="callback", type="string", metavar="DIR",
+            dest="pyidir", callback=store_abspath,
+            help="where the sip.pyi stub file will be installed [default: "
+                    "%s]" % plat_py_site_dir)
     p.add_option_group(g)
 
     return p
@@ -802,7 +826,7 @@ def main(argv):
     # Apply the overrides from any configuration file.
     global plat_bin_dir, plat_py_conf_inc_dir, plat_py_inc_dir
     global plat_py_lib_dir, plat_py_site_dir, plat_sip_dir
-    global sip_bin_dir, sip_inc_dir, sip_module_dir, sip_sip_dir
+    global sip_bin_dir, sip_inc_dir, sip_module_dir, sip_sip_dir, pyi_dir
 
     # Set defaults.
     sip_bin_dir = plat_bin_dir
@@ -847,6 +871,11 @@ def main(argv):
     if opts.sipsipdir is not None:
         sip_sip_dir = opts.sipsipdir
 
+    if opts.pyidir is not None:
+        pyi_dir = opts.pyidir
+    else:
+        pyi_dir = sip_module_dir
+
     # Get the platform specific macros for building.
     macros = siputils.parse_build_macros(
             os.path.join(src_dir, "specs", build_platform), build_macro_names,
